# > \brief \b SDSDOT
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def SDSDOT(N,SB,SX,INCX,SY,INCY)
#
#       .. Scalar Arguments ..
#       REAL SB
#       INTEGER INCX,INCY,N
#       ..
#       .. Array Arguments ..
#       REAL SX(*),SY(*)
#       ..
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# >   Compute the inner product of two vectors with extended
# >   precision accumulation.
# >
# >   Returns S.P. result with dot product accumulated in D.P.
# >   SDSDOT = SB + sum for I = 0 to N-1 of SX(LX+I*INCX)*SY(LY+I*INCY),
# >   where LX = 1 if INCX >= 0, else LX = 1+(1-N)*INCX, and LY is
# >   defined in a similar way using INCY.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] N
# > \verbatim
# >          N is INTEGER
# >          number of elements in input vector(s)
# > \endverbatim
# >
# > \param[in] SB
# > \verbatim
# >          SB is REAL
# >          single precision scalar to be added to inner product
# > \endverbatim
# >
# > \param[in] SX
# > \verbatim
# >          SX is REAL array, dimension ( 1 + ( N - 1 )*abs( INCX ) )
# >          single precision vector with N elements
# > \endverbatim
# >
# > \param[in] INCX
# > \verbatim
# >          INCX is INTEGER
# >          storage spacing between elements of SX
# > \endverbatim
# >
# > \param[in] SY
# > \verbatim
# >          SY is REAL array, dimension ( 1 + ( N - 1 )*abs( INCX ) )
# >          single precision vector with N elements
# > \endverbatim
# >
# > \param[in] INCY
# > \verbatim
# >          INCY is INTEGER
# >          storage spacing between elements of SY
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Lawson, C. L., (JPL), Hanson, R. J., (SNLA),
# > \author Kincaid, D. R., (U. of Texas), Krogh, F. T., (JPL)
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup single_blas_level1
#
# > \par Further Details:
#  =====================
# >
# > \verbatim
# >
# >    REFERENCES
# >
# >    C. L. Lawson, R. J. Hanson, D. R. Kincaid and F. T.
# >    Krogh, Basic linear algebra subprograms for Fortran
# >    usage, Algorithm No. 539, Transactions on Mathematical
# >    Software 5, 3 (September 1979), pp. 308-323.
# >
# >    REVISION HISTORY  (YYMMDD)
# >
# >    791001  DATE WRITTEN
# >    890531  Changed all specific intrinsics to generic.  (WRB)
# >    890831  Modified array declarations.  (WRB)
# >    890831  REVISION DATE from Version 3.2
# >    891214  Prologue converted to Version 4.0 format.  (BAB)
# >    920310  Corrected definition of LX in DESCRIPTION.  (WRB)
# >    920501  Reformatted the REFERENCES section.  (WRB)
# >    070118  Reformat to LAPACK coding style
# > \endverbatim
# >
#  =====================================================================
def SDSDOT(N, SB, SX, INCX, SY, INCY):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # REAL SB
    # INTEGER INCX,INCY,N
    #     ..
    #     .. Array Arguments ..
    # REAL SX(*),SY(*)
    #     .. Local Scalars ..
    # DOUBLE PRECISION DSDOT
    # INTEGER I,KX,KY,NS
    #     ..
    #     .. Intrinsic Functions ..
    # INTRINSIC DBLE
    #     ..
    DSDOT = SB
    if N <= 0:
        return DSDOT
    if INCX == INCY and INCX > 0:
        #
        #     Code for equal and positive increments.
        #
        NS = N * INCX
        for I in range(0, NS, INCX):
            DSDOT += (SX[I]).real * (SY[I]).rel
    else:
        #
        #     Code for unequal or nonpositive increments.
        #
        KX = 1
        KY = 1
        if INCX < 0:
            KX = 1 + (1 - N) * INCX
        if INCY < 0:
            KY = 1 + (1 - N) * INCY
        for I in range(N):
            DSDOT += (SX[KX]).real * (SY[KY]).real
            KX += INCX
            KY += INCY
    return DSDOT



# > \brief \b DROTMG
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def DROTMG(DD1,DD2,DX1,DY1,DPARAM)
#
#       .. Scalar Arguments ..
#       DOUBLE PRECISION DD1,DD2,DX1,DY1
#       ..
#       .. Array Arguments ..
#       DOUBLE PRECISION DPARAM(5)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# >    CONSTRUCT THE MODIFIED GIVENS TRANSFORMATION MATRIX H WHICH ZEROS
# >    THE SECOND COMPONENT OF THE 2-VECTOR  (sqrt(DD1)*DX1,sqrt(DD2)*>    DY2)**T.
# >    WITH DPARAM(1)=DFLAG, H HAS ONE OF THE FOLLOWING FORMS..
# >
# >    DFLAG=-1.D0     DFLAG=0.D0        DFLAG=1.D0     DFLAG=-2.D0
# >
# >      (DH11  DH12)    (1.D0  DH12)    (DH11  1.D0)    (1.D0  0.D0)
# >    H=(          )    (          )    (          )    (          )
# >      (DH21  DH22),   (DH21  1.D0),   (-1.D0 DH22),   (0.D0  1.D0).
# >    LOCATIONS 2-4 OF DPARAM CONTAIN DH11, DH21, DH12, AND DH22
# >    RESPECTIVELY. (VALUES OF 1.D0, -1.D0, OR 0.D0 IMPLIED BY THE
# >    VALUE OF DPARAM(1) ARE NOT STORED IN DPARAM.)
# >
# >    THE VALUES OF GAMSQ AND RGAMSQ SET IN THE DATA STATEMENT MAY BE
# >    INEXACT.  THIS IS OK AS THEY ARE ONLY USED FOR TESTING THE SIZE
# >    OF DD1 AND DD2.  ALL ACTUAL SCALING OF DATA IS DONE USING GAM.
# >
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in,out] DD1
# > \verbatim
# >          DD1 is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[in,out] DD2
# > \verbatim
# >          DD2 is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[in,out] DX1
# > \verbatim
# >          DX1 is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[in] DY1
# > \verbatim
# >          DY1 is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[out] DPARAM
# > \verbatim
# >          DPARAM is DOUBLE PRECISION array, dimension (5)
# >     DPARAM(1)=DFLAG
# >     DPARAM(2)=DH11
# >     DPARAM(3)=DH21
# >     DPARAM(4)=DH12
# >     DPARAM(5)=DH22
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup double_blas_level1
#
#  =====================================================================
def DROTMG(DD1, DD2, DX1, DY1, DPARAM):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # DOUBLE PRECISION DD1,DD2,DX1,DY1
    #     ..
    #     .. Array Arguments ..
    # DOUBLE PRECISION DPARAM(5)
    #     ..
    #
    #  =====================================================================
    #
    #     .. Local Scalars ..
    #    DOUBLE PRECISION DFLAG,DH11,DH12,DH21,DH22,DP1,DP2,DQ1,DQ2,DTEMP,
    #   $                 DU,GAM,GAMSQ,ONE,RGAMSQ,TWO,ZERO
    #     ..
    #     .. Intrinsic Functions ..
    # INTRINSIC DABS
    #     ..
    #     .. Data statements ..
    #
    # DATA ZERO,ONE,TWO/0.D0,1.D0,2.D0/
    #  DATA GAM,GAMSQ,RGAMSQ/4096.D0,16777216.D0,5.9604645D-8/
    #     ..
    GAM = 4096
    GAMSQ = 16777216
    RGAMSQ = 5.9604645e-8
    if DD1 < 0:
        #        GO ZERO-H-D-AND-DX1..
        DFLAG = -1
        DH11 = 0
        DH12 = 0
        DH21 = 0
        DH22 = 0
        #
        DD1 = 0
        DD2 = 0
        DX1 = 0
    else:
        #        CASE-DD1-NONNEGATIVE
        DP2 = DD2 * DY1
        if DP2 == 0:
            DFLAG = -2
            DPARAM[1] = DFLAG
            return
        #        REGULAR-CASE..
        DP1 = DD1 * DX1
        DQ2 = DP2 * DY1
        DQ1 = DP1 * DX1
        #
        if abs(DQ1) > abs(DQ2):
            DH21 = -DY1 / DX1
            DH12 = DP2 / DP1
            #
            DU = 1 - DH12 * DH21
            #
            if DU > 0:
                DFLAG = 0
                DD1 = DD1 / DU
                DD2 = DD2 / DU
                DX1 = DX1 * DU
        else:

            if DQ2 < 0:
                #               GO ZERO-H-D-AND-DX1..
                DFLAG = -1
                DH11 = 0
                DH12 = 0
                DH21 = 0
                DH22 = 0
                #
                DD1 = 0
                DD2 = 0
                DX1 = 0
            else:
                DFLAG = 1
                DH11 = DP1 / DP2
                DH22 = DX1 / DY1
                DU = 1 + DH11 * DH22
                DTEMP = DD2 / DU
                DD2 = DD1 / DU
                DD1 = DTEMP
                DX1 = DY1 * DU

        # PROCEDURE..SCALE-CHECK
        if DD1 != 0:
            while (DD1 <= RGAMSQ) or (DD1 >= GAMSQ):
                if DFLAG == 0:
                    DH11 = 1
                    DH22 = 1
                    DFLAG = -1
                else:
                    DH21 = -1
                    DH12 = 1
                    DFLAG = -1
                if DD1 <= RGAMSQ:
                    DD1 = DD1 * GAM ** 2
                    DX1 = DX1 / GAM
                    DH11 = DH11 / GAM
                    DH12 = DH12 / GAM
                else:
                    DD1 = DD1 / GAM ** 2
                    DX1 = DX1 * GAM
                    DH11 = DH11 * GAM
                    DH12 = DH12 * GAM

        if DD2 != 0:
            while (abs(DD2) <= RGAMSQ) or (abs(DD2) >= GAMSQ):
                if DFLAG == 0:
                    DH11 = 1
                    DH22 = 1
                    DFLAG = -1
                else:
                    DH21 = -1
                    DH12 = 1
                    DFLAG = -1
                if abs(DD2) <= RGAMSQ:
                    DD2 = DD2 * GAM ** 2
                    DH21 = DH21 / GAM
                    DH22 = DH22 / GAM
                else:
                    DD2 = DD2 / GAM ** 2
                    DH21 = DH21 * GAM
                    DH22 = DH22 * GAM

    if DFLAG < 0:
        DPARAM[2] = DH11
        DPARAM[3] = DH21
        DPARAM[4] = DH12
        DPARAM[5] = DH22
    elif DFLAG == 0:
        DPARAM[3] = DH21
        DPARAM[4] = DH12
    else:
        DPARAM[2] = DH11
        DPARAM[5] = DH22

    DPARAM[1] = DFLAG



# > \brief \b CDOTC
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def CDOTC(N,CX,INCX,CY,INCY)
#
#       .. Scalar Arguments ..
#       INTEGER INCX,INCY,N
#       ..
#       .. Array Arguments ..
#       COMPLEX CX(*),CY(*)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# > CDOTC forms the dot product of two complex vectors
# >      CDOTC = X^H * Y
# >
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] N
# > \verbatim
# >          N is INTEGER
# >         number of elements in input vector(s)
# > \endverbatim
# >
# > \param[in] CX
# > \verbatim
# >          CX is COMPLEX array, dimension ( 1 + ( N - 1 )*abs( INCX ) )
# > \endverbatim
# >
# > \param[in] INCX
# > \verbatim
# >          INCX is INTEGER
# >         storage spacing between elements of CX
# > \endverbatim
# >
# > \param[in] CY
# > \verbatim
# >          CY is COMPLEX array, dimension ( 1 + ( N - 1 )*abs( INCY ) )
# > \endverbatim
# >
# > \param[in] INCY
# > \verbatim
# >          INCY is INTEGER
# >         storage spacing between elements of CY
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup complex_blas_level1
#
# > \par Further Details:
#  =====================
# >
# > \verbatim
# >
# >     jack dongarra, linpack,  3/11/78.
# >     modified 12/3/93, array(1) declarations changed to array(*)
# > \endverbatim
# >
#  =====================================================================
def cdotc(N, CX, INCX, CY, INCY):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # INTEGER INCX,INCY,N
    #     ..
    #     .. Array Arguments ..
    # COMPLEX CX(*),CY(*)
    #     ..
    #
    #  =====================================================================
    CTEMP = 0
    if N <= 0:
        return 0
    if INCX == 1 and INCY == 1:
        # code for both increments equal to 1
        for I in range(N):
            CTEMP += CX[I].conjugate() * CY[I]
    else:
        # code for unequal increments or equal increments not equal to 1
        IX = 1
        IY = 1
        if INCX < 0:
            IX = (-N + 1) * INCX + 1
        if INCY < 0:
            IY = (-N + 1) * INCY + 1
        for I in range(N):
            CTEMP += CX[IX].conjugate() * CY[IY]
            IX += INCX
            IY += INCY
    return CTEMP



# > \brief \b SROTM
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def SROTM(N,SX,INCX,SY,INCY,SPARAM)
#
#       .. Scalar Arguments ..
#       INTEGER INCX,INCY,N
#       ..
#       .. Array Arguments ..
#       REAL SPARAM(5),SX(*),SY(*)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# >    APPLY THE MODIFIED GIVENS TRANSFORMATION, H, TO THE 2 BY N MATRIX
# >
# >    (SX**T) , WHERE **T INDICATES TRANSPOSE. THE ELEMENTS OF SX ARE IN
# >    (SX**T)
# >
# >    SX(LX+I*INCX), I = 0 TO N-1, WHERE LX = 1 IF INCX >= 0, else:
# >    LX = (-INCX)*N, AND SIMILARLY FOR SY USING USING LY AND INCY.
# >    WITH SPARAM(1)=SFLAG, H HAS ONE OF THE FOLLOWING FORMS..
# >
# >    SFLAG=-1.E0     SFLAG=0.E0        SFLAG=1.E0     SFLAG=-2.E0
# >
# >      (SH11  SH12)    (1.E0  SH12)    (SH11  1.E0)    (1.E0  0.E0)
# >    H=(          )    (          )    (          )    (          )
# >      (SH21  SH22),   (SH21  1.E0),   (-1.E0 SH22),   (0.E0  1.E0).
# >    SEE  SROTMG FOR A DESCRIPTION OF DATA STORAGE IN SPARAM.
# >
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] N
# > \verbatim
# >          N is INTEGER
# >         number of elements in input vector(s)
# > \endverbatim
# >
# > \param[in,out] SX
# > \verbatim
# >          SX is REAL array, dimension ( 1 + ( N - 1 )*abs( INCX ) )
# > \endverbatim
# >
# > \param[in] INCX
# > \verbatim
# >          INCX is INTEGER
# >         storage spacing between elements of SX
# > \endverbatim
# >
# > \param[in,out] SY
# > \verbatim
# >          SY is REAL array, dimension ( 1 + ( N - 1 )*abs( INCY ) )
# > \endverbatim
# >
# > \param[in] INCY
# > \verbatim
# >          INCY is INTEGER
# >         storage spacing between elements of SY
# > \endverbatim
# >
# > \param[in] SPARAM
# > \verbatim
# >          SPARAM is REAL array, dimension (5)
# >     SPARAM(1)=SFLAG
# >     SPARAM(2)=SH11
# >     SPARAM(3)=SH21
# >     SPARAM(4)=SH12
# >     SPARAM(5)=SH22
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup single_blas_level1
#
#  =====================================================================
def SROTM(N, SX, INCX, SY, INCY, SPARAM):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # INTEGER INCX,INCY,N
    #     ..
    #     .. Array Arguments ..
    # REAL SPARAM(5),SX(*),SY(*)
    #     ..
    #
    #  =====================================================================
    #
    #     .. Local Scalars ..
    # REAL SFLAG,SH11,SH12,SH21,SH22,TWO,W,Z,ZERO
    # INTEGER I,KX,KY,NSTEPS
    #     ..
    #     .. Data statements ..
    # DATA ZERO,TWO/0.E0,2.E0/
    #     ..
    #
    SFLAG = SPARAM(1)
    if N <= 0 or (SFLAG + 2 == 0):
        return
    if INCX == INCY and INCX > 0:
        #
        NSTEPS = N * INCX
        if SFLAG < 0:
            SH11 = SPARAM[2]
            SH12 = SPARAM[4]
            SH21 = SPARAM[3]
            SH22 = SPARAM[5]
            for I in range(0, NSTEPS, INCX):
                W = SX[I]
                Z = SY(I)
                SX[I] = W * SH11 + Z * SH12
                SY[I] = W * SH21 + Z * SH22
        elif SFLAG == 0:
            SH12 = SPARAM(4)
            SH21 = SPARAM(3)
            for I in range(0, NSTEPS, INCX):
                W = SX[I]
                Z = SY[I]
                SX[I] = W + Z * SH12
                SY[I] = W * SH21 + Z
        else:
            SH11 = SPARAM(2)
            SH22 = SPARAM(5)
            for I in range(0, NSTEPS, INCX):
                W = SX[I]
                Z = SY[I]
                SX[I] = W * SH11 + Z
                SY[I] = -W + SH22 * Z
    else:
        KX = 1
        KY = 1
        if INCX < 0:
            KX = 1 + (1 - N) * INCX
        if INCY < 0:
            KY = 1 + (1 - N) * INCY
        #
        if SFLAG < 0:
            SH11 = SPARAM(2)
            SH12 = SPARAM(4)
            SH21 = SPARAM(3)
            SH22 = SPARAM(5)
            for I in range(N):
                W = SX[KX]
                Z = SY[KY]
                SX[KX] = W * SH11 + Z * SH12
                SY[KY] = W * SH21 + Z * SH22
                KX += INCX
                KY += INCY
        elif SFLAG == 0:
            SH12 = SPARAM(4)
            SH21 = SPARAM(3)
            for I in range(N):
                W = SX[KX]
                Z = SY[KY]
                SX[KX] = W + Z * SH12
                SY[KY] = W * SH21 + Z
                KX += INCX
                KY += INCY
        else:
            SH11 = SPARAM(2)
            SH22 = SPARAM(5)
            for I in range(N):
                W = SX[KX]
                Z = SY[KY]
                SX[KX] = W * SH11 + Z
                SY[KY] = -W + SH22 * Z
                KX += INCX
                KY += INCY



# > \brief \b SAXPY
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def SAXPY(N,SA,SX,INCX,SY,INCY)
#
#       .. Scalar Arguments ..
#       REAL SA
#       INTEGER INCX,INCY,N
#       ..
#       .. Array Arguments ..
#       REAL SX(*),SY(*)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# >    SAXPY constant times a vector plus a vector.
# >    uses unrolled loops for increments equal to one.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] N
# > \verbatim
# >          N is INTEGER
# >         number of elements in input vector(s)
# > \endverbatim
# >
# > \param[in] SA
# > \verbatim
# >          SA is REAL
# >           On entry, SA specifies the scalar alpha.
# > \endverbatim
# >
# > \param[in] SX
# > \verbatim
# >          SX is REAL array, dimension ( 1 + ( N - 1 )*abs( INCX ) )
# > \endverbatim
# >
# > \param[in] INCX
# > \verbatim
# >          INCX is INTEGER
# >         storage spacing between elements of SX
# > \endverbatim
# >
# > \param[in,out] SY
# > \verbatim
# >          SY is REAL array, dimension ( 1 + ( N - 1 )*abs( INCY ) )
# > \endverbatim
# >
# > \param[in] INCY
# > \verbatim
# >          INCY is INTEGER
# >         storage spacing between elements of SY
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup single_blas_level1
#
# > \par Further Details:
#  =====================
# >
# > \verbatim
# >
# >     jack dongarra, linpack, 3/11/78.
# >     modified 12/3/93, array(1) declarations changed to array(*)
# > \endverbatim
# >
#  =====================================================================
def SAXPY(N, SA, SX, INCX, SY, INCY):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # REAL SA
    # INTEGER INCX,INCY,N
    #     ..
    #     .. Array Arguments ..
    # REAL SX(*),SY(*)
    #     ..
    #
    #  =====================================================================
    #
    #     .. Local Scalars ..
    # INTEGER I,IX,IY,M,MP1
    #     ..
    #     .. Intrinsic Functions ..
    # INTRINSIC MOD
    #     ..
    if N <= 0:
        return
    if SA == 0.0:
        return
    if INCX == 1 and INCY == 1:
        # code for both increments equal to 1

        # clean-up loop
        M = N % 4
        if M != 0:
            for I in range(M):
                SY[I] += SA * SX[I]
        if N < 4:
            return
        for I in range(M, N, 4):
            SY[I] += SA * SX[I]
            SY[I + 1] += SA * SX[I + 1]
            SY[I + 2] += SA * SX[I + 2]
            SY[I + 3] += SA * SX[I + 3]
    else:
        # code for unequal increments or equal increments not equal to 1
        IX = 1
        IY = 1
        if INCX < 0:
            IX = (-N + 1) * INCX + 1
        if INCY < 0:
            IY = (-N + 1) * INCY + 1
        for I in range(N):
            SY[IY] += SA * SX[IX]
            IX += INCX
            IY += INCY



# > \brief \b DZNRM2
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def DZNRM2(N,X,INCX)
#
#       .. Scalar Arguments ..
#       INTEGER INCX,N
#       ..
#       .. Array Arguments ..
#       COMPLEX*16 X(*)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# > DZNRM2 returns the euclidean norm of a vector via the function
# > name, so that
# >
# >    DZNRM2 := sqrt( x**H*x )
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] N
# > \verbatim
# >          N is INTEGER
# >         number of elements in input vector(s)
# > \endverbatim
# >
# > \param[in] X
# > \verbatim
# >          X is COMPLEX*16 array, dimension (N)
# >         complex vector with N elements
# > \endverbatim
# >
# > \param[in] INCX
# > \verbatim
# >          INCX is INTEGER
# >         storage spacing between elements of X
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup double_blas_level1
#
# > \par Further Details:
#  =====================
# >
# > \verbatim
# >
# >  -- This version written on 25-October-1982.
# >     Modified on 14-October-1993 to inline the call to ZLASSQ.
# >     Sven Hammarling, Nag Ltd.
# > \endverbatim
# >
#  =====================================================================
from math import sqrt


def DZNRM2(N, X, INCX):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    #   INTEGER INCX,N
    #     ..
    #     .. Array Arguments ..
    #   COMPLEX*16 X(*)
    #     ..
    #
    #  =====================================================================

    if N < 1 or INCX < 1:
        NORM = 0
    else:
        SCALE = 0
        SSQ = 1
        #        The following loop is equivalent to this call to the LAPACK
        #        auxiliary routine:
        #        CALL ZLASSQ( N, X, INCX, SCALE, SSQ )
        #
        for IX in range(0, 1 + (N - 1) * INCX, INCX):
            if (X[IX]).real != 0:
                TEMP = abs((X[IX]).real)
                if SCALE < TEMP:
                    SSQ = 1 + SSQ * (SCALE / TEMP) ** 2
                    SCALE = TEMP
                else:
                    SSQ = SSQ + (TEMP / SCALE) ** 2
            if (X[IX]).imag != 0:
                TEMP = abs((X[IX]).imag)
                if SCALE < TEMP:
                    SSQ = 1 + SSQ * (SCALE / TEMP) ** 2
                    SCALE = TEMP
                else:
                    SSQ = SSQ + (TEMP / SCALE) ** 2
        NORM = SCALE * sqrt(SSQ)

    return NORM



# > \brief \b CSCAL
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def CSCAL(N,CA,CX,INCX)
#
#       .. Scalar Arguments ..
#       COMPLEX CA
#       INTEGER INCX,N
#       ..
#       .. Array Arguments ..
#       COMPLEX CX(*)
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# >    CSCAL scales a vector by a constant.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] N
# > \verbatim
# >          N is INTEGER
# >         number of elements in input vector(s)
# > \endverbatim
# >
# > \param[in] CA
# > \verbatim
# >          CA is COMPLEX
# >           On entry, CA specifies the scalar alpha.
# > \endverbatim
# >
# > \param[in,out] CX
# > \verbatim
# >          CX is COMPLEX array, dimension ( 1 + ( N - 1 )*abs( INCX ) )
# > \endverbatim
# >
# > \param[in] INCX
# > \verbatim
# >          INCX is INTEGER
# >         storage spacing between elements of CX
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup complex_blas_level1
#
# > \par Further Details:
#  =====================
# >
# > \verbatim
# >
# >     jack dongarra, linpack,  3/11/78.
# >     modified 3/93 to return if incx <= 0.
# >     modified 12/3/93, array(1) declarations changed to array(*)
# > \endverbatim
# >
#  =====================================================================
def cscal(N, CA, CX, INCX):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # COMPLEX CA
    # INTEGER INCX,N
    #     ..
    #     .. Array Arguments ..
    # COMPLEX CX(*)
    #     ..
    #
    #  =====================================================================
    if N <= 0 or INCX <= 0:
        return
    if INCX == 1:
        # code for increment equal to 1
        for I in range(N):
            CX[I] = CA * CX[I]
    else:
        # code for increment not equal to 1
        for I in range(0, N * INCX, INCX):
            CX[I] = CA * CX[I]



# > \brief \b DROTG
#
#  =========== DOCUMENTATION ===========
#
# Online html documentation available at
#            http://www.netlib.org/lapack/explore-html/
#
#  Definition:
#  ===========
#
#       def DROTG(DA,DB,C,S)
#
#       .. Scalar Arguments ..
#       DOUBLE PRECISION C,DA,DB,S
#       ..
#
#
# > \par Purpose:
#  =============
# >
# > \verbatim
# >
# >    DROTG construct givens plane rotation.
# > \endverbatim
#
#  Arguments:
#  ==========
#
# > \param[in] DA
# > \verbatim
# >          DA is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[in] DB
# > \verbatim
# >          DB is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[out] C
# > \verbatim
# >          C is DOUBLE PRECISION
# > \endverbatim
# >
# > \param[out] S
# > \verbatim
# >          S is DOUBLE PRECISION
# > \endverbatim
#
#  Authors:
#  ========
#
# > \author Univ. of Tennessee
# > \author Univ. of California Berkeley
# > \author Univ. of Colorado Denver
# > \author NAG Ltd.
#
# > \date November 2017
#
# > \ingroup double_blas_level1
#
# > \par Further Details:
#  =====================
# >
# > \verbatim
# >
# >     jack dongarra, linpack, 3/11/78.
# > \endverbatim
# >
#  =====================================================================
from math import sqrt
from util import sign


def DROTG(DA, DB, C, S):
    #
    #  -- Reference BLAS level1 routine (version 3.8.0) --
    #  -- Reference BLAS is a software package provided by Univ. of Tennessee,    --
    #  -- Univ. of California Berkeley, Univ. of Colorado Denver and NAG Ltd..--
    #     November 2017
    #
    #     .. Scalar Arguments ..
    # DOUBLE PRECISION C,DA,DB,S
    #     ..
    #
    #  =====================================================================
    #
    #     .. Local Scalars ..
    # DOUBLE PRECISION R,ROE,SCALE,Z
    #     ..
    #     .. Intrinsic Functions ..
    # INTRINSIC DABS,DSIGN,DSQRT
    #     ..
    ROE = DB
    if abs(DA) > abs(DB):
        ROE = DA
    SCALE = abs(DA) + abs(DB)
    if SCALE == 0:
        C = 1
        S = 0
        R = 0
        Z = 0
    else:
        R = SCALE * sqrt((DA / SCALE) ** 2 + (DB / SCALE) ** 2)
        R = sign(ROE) * R
        C = DA / R
        S = DB / R
        Z = 1
        if abs(DA) > abs(DB):
            Z = S
        if abs(DB) >= abs(DA) and C != 0:
            Z = 1 / C
    DA = R
    DB = Z
